from .datastorage import DataStorage
from .utils import get_dataset_property


class Minio(DataStorage):

    # overriding abstract method
    def load_dataset(self, name, read_all=True, input_dest='', header=True):

        bucket = get_dataset_property(name, 'minio_bucket')

        if input_dest == '':
            input_dataset = get_dataset_property(name)
        else:
            input_dataset = input_dest
        
        delimiter = get_dataset_property(name, "delimiter") if get_dataset_property(name, "delimiter") is not None else ","
        
        input_file_format = get_dataset_property(name, "inputFileFormat") if get_dataset_property(name, "inputFileFormat") is not None else "csv"
        
        input_columns = get_dataset_property(name, "input_columns")
        if input_columns is not None:
            columns_list = input_columns.split(",")
        else:
            columns_list = ["*"]
        
        if read_all is True:
            return self.spark.read.load("s3a://" + bucket + "/" + input_dataset, format=input_file_format, sep=delimiter, inferSchema=True, header=header, error_bad_lines=False)
        else:
            return self.spark.read.load("s3a://" + bucket + "/" + input_dataset, format=input_file_format, sep=delimiter, inferSchema=True, header=header, error_bad_lines=False).select(*columns_list)

    # overriding abstract method
    def save_dataset(self, name, df, output_dest):

        bucket = get_dataset_property(name, 'minio_bucket')

        if output_dest == '':
            output_dataset = get_dataset_property(name)
        else:
            output_dataset = output_dest

        delimiter = get_dataset_property(name, "delimiter") if get_dataset_property(name, "delimiter") is not None else ","
        output_file_format = get_dataset_property(name, "outputFileFormat") if get_dataset_property(name, "outputFileFormat") is not None else "csv"

        df.write.options(delimiter=delimiter).\
            mode('overwrite').\
            option("header", True).\
            save("s3a://" + bucket + "/" + output_dataset, format = output_file_format)



from .datastorage import DataStorage
from .utils import get_dataset_property
import os


class Hive(DataStorage):

    # overriding abstract method
    def load_dataset(self, name, read_all=True, input_dest='', header=True):

        input_columns = get_dataset_property(name, "input_columns")

        if input_dest == '':
            input_dataset = get_dataset_property(name)
        else:
            input_dataset = input_dest

        if read_all:
            df = self.spark.sql("SELECT * FROM " + input_dataset)
        else:
            df = self.spark.sql("SELECT " + input_columns + " FROM " + input_dataset)

        # rename column
        for column in input_columns.split(","):
            df = df.withColumnRenamed(column.lower(), column)
        return df

    # overriding abstract method
    def save_dataset(self, name, df, output_dest):

        if output_dest == '':
            output_dataset = get_dataset_property(name)
        else:
            output_dataset = output_dest

        os.environ["HADOOP_USER_NAME"] = get_dataset_property(name, "hiveHadoopUserName")

        database_name = output_dataset.split(".")[0]
        self.spark.sql("CREATE DATABASE IF NOT EXISTS " + database_name)
        df.write.mode("overwrite").saveAsTable(output_dataset)



from alidaargparser import get_asset_property

def input_or_output(name):
    return get_asset_property(name, property="direction")

def get_dataset_property(name, prop=None):
    return get_asset_property(asset_name=name, property=prop)    



from abc import ABC


class DataStorage(ABC):

    def __init__(self, spark):

        self.spark = spark

    # abstract method
    def load_dataset(self, read_all=True):
        pass

    # abstract method
    def save_dataset(self, df, output_dest):
        pass



from .utils import get_dataset_property
import importlib



def get_class(module_name):

    module = importlib.import_module("." + module_name, "pysparkutilities")
    return getattr(module, module_name.capitalize())


def load_dataset(sc, name, read_all, input_dest='', header=True):
    # sc -> Spark context
    # read_all -> True to read all data, False to read 'input_columns' in 'args'

    data_storage = get_dataset_property(name=name, prop='storage_type')
    module_name = data_storage.split('-')[0].lower()

    class_ = get_class(module_name)
    instance = class_(sc)

    return instance.load_dataset(name, read_all, input_dest, header)


def save_dataset(sc, name, df, output_dest=''):
    # df -> Spark dataframe to store
    # output_dest -> destinaion output, if not specified it takes 'output-dataset' destinaion from 'args'

    data_storage = get_dataset_property(name=name, prop='storage_type') 
    module_name = data_storage.split('-')[0].lower()

    class_ = get_class(module_name)
    instance = class_(sc)

    return instance.save_dataset(name, df, output_dest)


def lower_columns_list(columns_list):

    return [x.lower() for x in columns_list]
